#include <stdint.h>
#include "lcd.h"
// #include "vpmem.h"
uint16_t mem[256];
uint8_t ppmem_num_banks;

void ppmem_init() {
  for (uint8_t i=0;i<2;i++) {
    mem[i]=i;
  }
  ppmem_num_banks=32;
  for (uint8_t i=2;i<ppmem_num_banks;i++) {
    mem[i]=0xFFFF;
  }
  for (uint16_t i=ppmem_num_banks;i<256;i++) {
    mem[i]=0xFFFE;
  }
}

uint16_t ppmem_to_vpmem(uint8_t page) {
  uint16_t frame=mem[page];
  if (frame==0xFFFE) {
    frame=0xFFFF;
  }
  return frame;
}

uint16_t ppmem_free(uint8_t start, uint8_t length) {
  uint16_t vpmem_start=mem[start];
  for (uint8_t i=0;i<length;i++) {
    mem[i+start]=0xFFFF;
  }
  return vpmem_start;
}

int16_t ppmem_map(uint16_t vpmem_start,uint8_t length) {
  int16_t ppmem_exist_start=-1;
  for (uint8_t i2=0;i2<255;i2++) {
    if (mem[i2]==vpmem_start) {
      ppmem_exist_start=i2;
    }
  }
  if (ppmem_exist_start!=-1) {
    uint8_t is_ok_length=1;
    for (uint8_t i=0;i<length;i++) {
      if (mem[ppmem_exist_start+i]!=vpmem_start+i) {
        is_ok_length=0;
        break;
      }
    }
    if (is_ok_length) {
      return ppmem_exist_start;
    }
  }
  if (length>ppmem_num_banks-2) {
    return -1;
  }
  int16_t ppmem_index=-1;
  uint16_t remaining_blks=length;
  uint8_t got_0=0;
  uint16_t old_i=0;
  for(uint8_t i=2;i<ppmem_num_banks;i++) {
    uint8_t bit=(mem[i]!=0xFFFF);
    if (got_0) {
      if (bit) {
        if (remaining_blks==0) {
            ppmem_index=old_i;
            break;
        } else {
          remaining_blks=length;
          got_0=0;
        }
      } else {
        remaining_blks--;
        if (remaining_blks==0) {
          ppmem_index=old_i;
          break;
        }
      }
    } else {
      if (!bit) {
        got_0=1;
        old_i=i;
        remaining_blks--;
        if (remaining_blks==0) {
          ppmem_index=old_i;
          break;
        }
      }
    }
  }
  if (remaining_blks!=0) {
    return 0x0;
  }
  for (uint8_t i=0;i<length;i++) {
    mem[i+ppmem_index]=vpmem_start+i;
  }
  return ppmem_index;
}
