#include <stdint.h>
#include "vpmem.h"
#include "ppmem.h"
#include "tasking.h"
#include "tasking_helpers.h"
#include "lcd.h"

uint8_t task_bmap[128];

#define NUM_TASK_BANKS

#ifndef LINT
__sfr __at 0x0 bank0;
#endif

#ifdef LINT
  char bank0;
#endif

uint16_t tasking_vpmem_bank;

int16_t head_task;
int16_t end_task;
int16_t current_task;

static uint8_t get_bmap_bit(uint16_t bit) {
  uint8_t bit_no=bit%8;
  //bank0=bitmap_banks[bit>>17];
  uint8_t byte=task_bmap[bit/8];
  return (byte&(1<<bit_no))>>bit_no;
}

static void set_bmap_bit(uint16_t bit) {
  uint16_t byte_no=bit>>3;
  uint8_t byte=task_bmap[byte_no];
  byte=byte|(1<<(bit&0x7));
  task_bmap[byte_no]=byte;
}

static void clear_bmap_bit(uint16_t bit) {
  uint16_t byte_no=bit>>3;
  uint8_t byte=task_bmap[byte_no];
  byte=byte&(~(1<<(bit&0x7)));
  task_bmap[byte_no]=byte;
}

static int16_t alloc_task() {
  for(uint16_t i=0;i<1024;i++) {
    if (get_bmap_bit(i)==0) {
      set_bmap_bit(i);
      return i;
    }
  }
  return -1;
}

void tasking_new(void* pc,uint16_t bank_table[4]) {
  // lcd_printf("VPMEM bank is %d\n",tasking_vpmem_bank);
  // uint8_t tasking_ppmem_bank=ppmem_map(tasking_vpmem_bank,1);
  // lcd_printf("Bank is %d\n",tasking_ppmem_bank);
  // bank0=tasking_ppmem_bank;
  bank0=(uint8_t)tasking_vpmem_bank;
  int16_t task_no=alloc_task();
  Task* addr=(Task*)(task_no*16);
  addr->pc_or_sp=(uint16_t)pc;
  for (uint8_t i=0;i<4;i++) {
    addr->bank_table[i]=bank_table[i];
  }
  addr->flags=1;
  addr->next_task=-1;
  if (head_task==-1) {
    addr->prev_task=-1;
    addr->flags=0;
    head_task=task_no;
    end_task=task_no;
    current_task=task_no;
  } else {
    Task* end_addr=(Task*)(end_task*16);
    end_addr->next_task=task_no;
    addr->prev_task=end_task;
    end_task=task_no;
  }
}

void tasking_yield() {
  if (current_task==-1) {
    return;
  }
  Task* addr=(Task*)(current_task*16);
  int16_t next_task=addr->next_task;
  if (next_task==-1) {
    if (current_task==head_task) {
      return;
    }
    next_task=head_task;
  }
  Task* next_task_addr=(Task*)(next_task*16);
  Task* current_task_addr=(Task*)(current_task*16);
  tasking_switch_to_task(current_task_addr,next_task_addr);
}

void tasking_init() {
  tasking_vpmem_bank=vpmem_alloc(1);
  head_task=-1;
  end_task=-1;
  current_task=-1;
  for (uint8_t i=0;i<128;i++) {
    task_bmap[i]=0;
  }
  uint16_t bank_table[4]={0,0,0,0};
  tasking_new(0,bank_table);
}
